/* -*- mode: javascript; tab-width: 4; indent-tabs-mode: nil; -*- */

/**
 * Example thingy
 *
 * Now using the dirty small library.
 *
 * How-to:
 *
 * 1. Create a JavaScript file containing "(function(){BULK})();"
 *    where BULK is a catenation of the library, the soft-synth, this
 *    file, and *last of all* the main "on-load code".
 *
 * 2. <!-- To run in "debug mode", create an HTML file containing
 *    "<html><head /><body><script>JS</script></body></html>" where JS
 *    is the JavaScript produced in step 1. Open in a browser that
 *    happens to accept the beast. -->
 *
 * 3. To produce a minified "intro competition" version, use the
 *    provided GNU Makefile or other means (automatic, to keep sane)
 *    to do the following: Remove debugging code and other redundant
 *    functionality (lines that end with "DEBUG". Feed into a
 *    JavaScript minifier like Closure. Then pack with PNGinator or
 *    JSExe or similar.
 *
 *    TODO: Write some "batch file" for Windows users who don't want
 *    to install MinGW or similar GNU toolset.
 *
 **/

/*
The visuals of this production are synced to the following song
created using the great SoundBox minisynth:

http://sb.bitsnbites.eu/?data=U0JveAwC7dm9SsNQGMbx56RpBoeqizgGexWCuzeiWIoSxBIIRcwSQmiQlCKIeCdOjl6Nl6AnX5Dix5zo_5e-73vaLCfhTH1OD6SJfNc5izV6i2Ud7tp27BnnVc5040V2NXI9W3Z6jqNAS6Wdnm31lfouCL7f_6qat7rv-f6vFCnp9HSrZ71__7PZQrEWzYyrab9pHobzKEmivu__Qje66_RrndtT0_ZwwOd_qVyPX85_0cy1rY0AAAAAAAAAAPibulHZvtqoTH4ZlUmmuhrPT3oY8qPW-y__-C-qWKBYK1cZ8WVEAQAAAAAAAAAAAP_Sx8LYkpHrTetfnInkt7dNqV6m9vNb8S4BAAAAAAAAAAAwLHldJ-7OkV29jGXeU_mX4z23vGuarKxqZRr2Q6UkZQAAAAAAAAAAABiWTw

http://sb.bitsnbites.eu/?data=U0JveAwC7dk9SgNBGAbgd2MS0MKfSsvFnELwOJZCGhsRJN0SEiQhCCLexMrS03gE3c0aiKLWRp9n-L4ZdqaY2fa9Pky2U3Y7Z6NsPY9SO9qr20m_6DylM1j0L-tVO5JhrlKt9fGHPslvNxx-ff9J3ae5y82n87P3eV7XIgAAAAAAAPw16ynZQVYpWcomJUuK5Uge7nO7ya9s799kXrNlIjabZ5om3RtLwQAAAAAAAP6l14uirhTp9gftl85uUq62i0aqpPqp_EYAAAAAAAA2y7St0-7Ocb167KV4qVKe9_a7zW7RxmTLIOybqoRkAAAAAAAAbJY3

http://sb.bitsnbites.eu/?data=U0JveAwC7dsxSgNBGAXgN0lcBUGbCJYL3kO8iY1HsLFLt4ghiEsgbJPz2NrbeY842WCZ3sD3wT_vzTA3GGZyk0zTTvO0SD7rPJ7NL5L8XJZJl7R989KUaplV3vORt7ymq22V5diHbNLXdbvNaBgOuV4fsu8DAAAAAAAA_8_uudRJyay5O5xMrpJ2X_bvY13NI9N1YwMAAAAAAIAT8r0o-Vqk7GbNfd0-nJfp7jrtPLfj_7H9lb9XsGMJAAAAAAAAp-IX

http://sb.bitsnbites.eu/?data=U0JveAwC7dwxSgQxFADQn9kxLgjarGAjDHgDDyAewt5mb6AgdtMNi-X2u9ewt7K1t_MI9uNkZ1EbtXXhveT__B9yg5BUxxHTaOqYtxHPQ1zvzaYR8XaQqi6iWeb7nAZRPGznt3VR5mLbr9fl1JiXY70aUolYbXoAAAAAAAD4R_rbNESkqPPZuFMdRjSl-Lwk636OrttUAAAAAAAAsENe2xQvbaS-zhdDe7mfJv1RNLM4-XpJVnR_rAAAAAAAALArqnkbk_LdYkQ-HdJTnSbnj3fvy3yVx7dkg5v4fQAAAAAAAMAu-QA

*/

// --------------------------------------------------------------------------------
// Variables used in this production, specifically:
var songBeatsPerMinute = 116;

// You can change/add whatever you want here:
var objTile, frames;

var count = 200;
// FIXME: Find a proper place for these:
clearColor=[0,0,0,0];
defaultLightDirection=[1,1,1,0];

// --------------------------------------------------------------------------------

/**
 * Initialize the constant and pre-computed "assets" used in your own
 * production; this function is called once before entering the main
 * loop.
 *
 * Things like graphics primitives / building blocks can be generated
 * here.
 */
function initAssets(){
    // Create primitive building blocks by different profile sweeps:

    objTile = new Box(1, 1, 1);

    frames = [];
    for (var t = 0; t < count; t++){
        var cube = new Box(1, Math.abs(1+Math.sin((t/count)*2*3.14)), 1);
        frames.push(cube);
    }
    fatframes = [];
    for (var t = 0; t < count; t++){
        var cube = new Box(1, Math.abs(1+10*Math.sin((t/count)*2*3.14)), 1);
        fatframes.push(cube);
    }

}




function snake(t, coloropt, cubecount = 20, fat = false){
    var cubes = [];
    var stuff = {
        f: [],
        o: [],
        c: []
    };
    var scale = .25;
    for (var i = 0; i<cubecount; i++) {
        var normal = ((1*(i/cubecount))*t)%1;
        // .9,.6,.4,1,

        var colorA=
            [.2,normal,.2,1,
             .5,.5,.5,1,
             0, 0, 0,1,
             2,1, 0,0];
        var colorB =
            [0.9,.8,normal,1,
             .5,.5,.5,1,
             0, 0, 0,1,
             2,1, 0,0];
        var colorC =
            [.1,.1,.1,1,
             .9,.1,normal/3,1,
             0, 0, 0,1,
             2,1, 0,0];
        var colorD =
            [.9,normal,.1,1,
             .5,.1,.1,1,
             0, 0, 0,1,
             2,1, 0,0];



        var modeframes = frames;
        if (fat == true) {modeframes = fatframes;}
        var cube = modeframes[Math.floor((((t*1000*0.25+i*100)%1000)/1000)*count)];
        var color = colorA;
        if (coloropt == 1){
            color = colorB;
        }
        if (coloropt == 2){
            color = colorC;
        }
        if (coloropt == 3){
            color = colorD;
        }

        stuff.c.push({f: [translate_wi(0,(i-cubecount)*2*scale,0), scaleXYZ_wi(scale, scale, scale)],
                  o: [new Material(color),cube],
                  c: []
                 });
    }
    return stuff;
}


/**
 * Your own creative input goes here - this function will be called on every screen update.
 *
 * Return a scene graph for a specific time. Time given as 'beats' according to song tempo.
 *
 * This is an important function to re-write creatively to make your own entry.
 *
 */
function buildSceneAtTime(t){

    // Initialize empty scenegraph. Root node with nothing inside:
    var sceneroot={f:[],o:[],c:[]};
    if (t > 73) return sceneroot;
    // Animation parameters
    //var stufftrans=[translate_wi(0,0,0),rotY_wi(-t*.8),rotX_wi(t*.02)];

    var sky =
            [100,100,100,1,
             0,0,0,1,
             0, 0, 0,1,
             2,1, 0,0];


    // var stuff = snowman(2*t);
     var snakeA  = snake(t, 0, 20, false);
     var snakeB = snake(t, 1, 20, false);
     var snakeC = snake(t, 2, 20, false);
     var snakeD = snake(t, 3, 40, true);
     sceneroot.c.push({f:[],
                       o:[],
                       c:[
                           {f:[translate_wi(0,0,0),scaleXYZ_wi(-100,-100,-100)],
                            o:[new Material(sky),objTile],
                            c:[]},

                           {f:[translate_wi(-1.5,3,0),scaleXYZ_wi(1,1,1)],
                            o:[],
                            c:[snakeA]},

                           {f:[translate_wi(1, 2.8,-1.5),scaleXYZ_wi(1,1,1), rotX_wi(80)],
                            o:[],
                            c:[snakeC]},


                           {f:[translate_wi(1.5,3,0),scaleXYZ_wi(1,1,1), rotX_wi(45)],
                            o:[],
                            c:[snakeB]},

                           {f:[translate_wi(-8,0,1),scaleXYZ_wi(1,1,1), rotX_wi(30)],
                            o:[],
                            c:[snakeD]},


                           {f:[translate_wi(0,3,0), rotY_wi(t*.16), translate_wi(0,0,20-10*Math.sin(t*.01)), rotX_wi(.2)],
                            o:[],
                            c:[],
                            r:[new Camera()]
                           }
                       ]
                      }
                     );

    return sceneroot;
}




// --------------------------------------------------------------------------------

/**
 * (Optionally) initialize additional HTML and CSS parts of the
 * document. This can be used, for example, for scrolling or flashing
 * text shown as usual HTML or hypertext. Not often used in actual
 * demoscene productions.
 */
function initDocument(){
}

/**
 * (Optionally) update the HTML and CSS parts of the document. This
 * can be used for scrolling or flashing text shown as usual HTML. Not
 * often used in actual demoscene productions.
 */
function updateDocument(t){
}
